/*
 * Meviz - Various tools collection to work with multimedia.
 * Copyright (C) 2012 -- 2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 */

package eu.svjatoslav.meviz.bomremove;

import eu.svjatoslav.commons.file.IOHelper;
import eu.svjatoslav.commons.string.WildCardMatcher;
import eu.svjatoslav.meviz.Module;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;

public class Main implements Module {

    private final byte[] bomHeader = new byte[]{(byte) 0xef, (byte) 0xbb, (byte) 0xbf};

    private final CommandlineHandler commandlineHandler = new CommandlineHandler();

    private BomStrippingOptions options;

    private boolean contains(final byte[] header, final byte[] patternToSeek) {

        for (int i = 0; i < patternToSeek.length; i++)
            if (header[i] != patternToSeek[i])
                return false;

        return true;
    }

    private boolean fileContainsHeader(final File file)
            throws IOException {

        final FileInputStream fileInputStream = new FileInputStream(file);

        final byte[] currentFileHeader = new byte[bomHeader.length];
        fileInputStream.read(currentFileHeader);
        fileInputStream.close();

        return contains(currentFileHeader, bomHeader);
    }

    private boolean fileMatchesInputPattern(final File file) {
        final String fileName = file.getName().toLowerCase();

        for (final String inputPattern : options.inputPatterns)
            if (WildCardMatcher.match(fileName, inputPattern.toLowerCase()))
                return true;

        return false;
    }

    @Override
    public String getDescription() {
        return "Remove byte order mark (bom) from UTF text files if they are present.";
    }

    @Override
    public String getModuleCommand() {
        return "stripbom";
    }

    private void processDirectory(final File directory) {

        for (final File subFile : directory.listFiles())
            if (subFile.isDirectory()) {
                if (options.recursive)
                    processDirectory(subFile);
            } else if (fileMatchesInputPattern(subFile))
                try {
                    processFile(subFile);
                } catch (final IOException exception) {
                    System.out.println("Error processing file: "
                            + subFile.getAbsolutePath());
                    System.out.println("   exception: "
                            + exception.getMessage());
                }

    }

    private void processFile(final File file) throws IOException {

        if (file.length() < bomHeader.length)
            return;

        if (!fileContainsHeader(file))
            return;

        System.out.println("Removing BOM from: " + file.getAbsolutePath());
        stripFileFromHeader(file);
    }

    @Override
    public void run(final String[] args) throws IOException {

        options = commandlineHandler.parseCommandlineArguments(args);

        if (options == null) {
            showCommandlineHelp();
            return;
        }

        processDirectory(options.targetDirectory);

    }

    @Override
    public void showCommandlineHelp() {
        commandlineHandler.parser.showHelp();
    }

    private void stripFileFromHeader(final File file)
            throws IOException {
        // read entire file
        final byte[] fileContents = IOHelper.getFileContents(file);

        // remove BOM header form file
        final byte[] newFileContents = Arrays.copyOfRange(fileContents,
                bomHeader.length, fileContents.length);

        // overwrite file with new contents
        IOHelper.saveToFile(file, newFileContents);
    }

}
