/*
 * Meviz - Various tools collection to work with multimedia.
 * Copyright (C) 2012 -- 2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 */

package eu.svjatoslav.meviz.encoder;

import eu.svjatoslav.commons.commandline.parameterparser.Parser;
import eu.svjatoslav.commons.commandline.parameterparser.parameter.DirectoryParameter;
import eu.svjatoslav.commons.commandline.parameterparser.parameter.IntegerParameter;
import eu.svjatoslav.commons.commandline.parameterparser.parameter.NullParameter;
import eu.svjatoslav.commons.commandline.parameterparser.parameter.StringParameters;

class CommandlineHandler {

    final Parser parser = new Parser();

    private final StringParameters outputFormatParameter = parser.add(new StringParameters("Encoding output format."))
            .addAliases("-o", "--output-format").setMandatory();

    private final IntegerParameter targetFramerate = parser
            .add(new IntegerParameter("Target framerate.").addAliases("-f", "--target-framerate"));

    private final NullParameter deinterlaceParameter = parser.add(new NullParameter("Deinterlace video.")).addAliases("-d",
            "--deinterlace");

    private final NullParameter testParameter = parser.add(new NullParameter("Simulate file encoding.")).addAliases("-t", "--test");

    private final NullParameter recursiveParameter = parser.add(new NullParameter("Enable recursive mode.")).addAliases("-r",
            "--recursive");

    private final NullParameter forPortableParameter = parser.add(new NullParameter("Encode for portable player.")).addAliases("-p",
            "--portable");

    private final StringParameters inputPatternParameter = parser.add(new StringParameters("File input pattern."))
            .addAliases("-i", "--input-pattern").setMandatory();

    private final DirectoryParameter workingDirectoryParameter = parser.add(new DirectoryParameter("Working directory."))
            .addAliases("-w", "--working-directory").mustExist();

    private final BitrateParameter videoBitrateParameter = parser.add(new BitrateParameter("Video bitrate")).addAliases("-v",
            "--video-bitrate");

    private final BitrateParameter audioBitrateParameter = parser.add(new BitrateParameter("Audio bitrate")).addAliases("-a",
            "--audio-bitrate");

    /**
     * @return {@link EncodingOptions} if commandline arguments were
     * successfully parsed, or <code>null</code> if parsing error
     * occurred.
     */
    public EncodingOptions parseCommandlineArguments(final String[] args) {

        final EncodingOptions options = new EncodingOptions();

        parser.parse(args);

        options.setRecursive(recursiveParameter.isSpecified());

        options.setDeinterlace(deinterlaceParameter.isSpecified());

        options.setTestOnly(testParameter.isSpecified());

        if (outputFormatParameter.isSpecified())
            options.getOutputFormats().addAll(outputFormatParameter.getValue());

        if (targetFramerate.isSpecified())
            options.setTargetFps(targetFramerate.getValue());

        if (workingDirectoryParameter.isSpecified())
            options.setWorkingDirectory(workingDirectoryParameter.getValue());

        if (inputPatternParameter.isSpecified())
            options.getInputPatterns().addAll(inputPatternParameter.getValue());

        options.setForPortablePlayer(forPortableParameter.isSpecified());

        if (videoBitrateParameter.isSpecified())
            options.setVideoBitrate(videoBitrateParameter.getValue());

        if (audioBitrateParameter.isSpecified())
            options.setAudioBitrate(audioBitrateParameter.getValue());

        return options;
    }
}
