package eu.svjatoslav.meviz.htmlindexer;

import eu.svjatoslav.meviz.htmlindexer.layouts.MixedLayout;
import eu.svjatoslav.meviz.htmlindexer.metadata.Dimension;
import eu.svjatoslav.meviz.htmlindexer.metadata.DirectoryMetadata;
import eu.svjatoslav.meviz.htmlindexer.metadata.fileTypes.AbstractFile;
import eu.svjatoslav.meviz.htmlindexer.metadata.fileTypes.Picture;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.util.List;

public class WebIndexer extends AbstractIndexer {

    private static final int METADATA_LOAD_TRY_COUNT = 10;
    private final String globalPrefix;
    private final String jspPath;

    public WebIndexer(final String globalPrefix, final String jspPath) {
        this.globalPrefix = globalPrefix;
        this.jspPath = jspPath;
    }

    @Override
    public void compileHtml(final Layout layout,
                            final DirectoryMetadata directory)
            throws UnsupportedEncodingException {
        super.compileHtml(layout, directory);
    }

    @Override
    public String getDirectoryUrl(final AbstractFile directory,
                                  final IndexingContext context) {
        return jspPath + context.getLocalUrl() + "/" + directory.fileName;
    }

    public String getHtml(String requestPath) throws
            IOException, ClassNotFoundException {

        if (requestPath == null)
            requestPath = "";

        if (requestPath.equals("/"))
            requestPath = "";

        final MixedLayout layout = new MixedLayout();
        final IndexingContext context = new IndexingContext(globalPrefix,
                requestPath);
        layout.init("Photos", context.getLocalPathComponents(), this, context);

        final DirectoryMetadata directory = getMetadataForPath(requestPath);

        compileHtml(layout, directory);

        return layout.getHtml(false, false, directory);
    }

    private DirectoryMetadata getMetadataForPath(final String requestPath)
            throws ClassNotFoundException, IOException {

        final String urlString = globalPrefix + Utils.urlEncode(requestPath) + "/.thumbnails/metadata_6.dat";

        for (int i = 0; true; i++)
            try {
                return attemptDirectoryMetadataDownload(urlString);
            } catch (final IOException e) {
                if (i > METADATA_LOAD_TRY_COUNT) throw e;
            }
    }

    private DirectoryMetadata attemptDirectoryMetadataDownload(String urlString) throws IOException, ClassNotFoundException {
        final BufferedInputStream inputStream = new BufferedInputStream(new URL(
                urlString).openStream());

        final ObjectInputStream objectInputStream = new ObjectInputStream(inputStream);
        final DirectoryMetadata directory = (DirectoryMetadata) objectInputStream
                .readObject();
        inputStream.close();

        return directory;
    }

    @Override
    public String getParentDirectoryUrl(final IndexingContext context) {
        final StringBuilder result = new StringBuilder();

        result.append(jspPath);

        final List<String> components = context.getLocalPathComponents();

        for (final String pathComponent : components.subList(0,
                components.size() - 1)) {
            result.append("/");
            result.append(pathComponent);
        }

        return result.toString();
    }

    @Override
    public String getThumbnailUrl(final Picture picture,
                                  final Dimension desiredDimension, final IndexingContext context) {

        // in case thumbnail size was equal to original, then return original
        // file path
        if (picture.getDimensions().equals(desiredDimension))
            return context.getGlobalUrl() + context.getLocalUrl() + "/"
                    + picture.fileName;

        final String thumbnailFileName = picture
                .getRelativeThumbnailFileName(desiredDimension);

        return context.getGlobalUrl() + context.getLocalUrl() + "/"
                + Constants.THUMBNAILS_DIRECTORY_NAME + "/"
                + Utils.urlEncode(thumbnailFileName);
    }

}
