/*
 * Meviz - Various tools collection to work with multimedia.
 * Copyright (C) 2012 -- 2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 */

package eu.svjatoslav.meviz.htmlindexer.layouts;

import eu.svjatoslav.commons.file.FilePathParser;
import eu.svjatoslav.meviz.htmlindexer.AbstractIndexer;
import eu.svjatoslav.meviz.htmlindexer.Constants;
import eu.svjatoslav.meviz.htmlindexer.IndexingContext;
import eu.svjatoslav.meviz.htmlindexer.Layout;
import eu.svjatoslav.meviz.htmlindexer.metadata.Dimension;
import eu.svjatoslav.meviz.htmlindexer.metadata.DirectoryMetadata;
import eu.svjatoslav.meviz.htmlindexer.metadata.fileTypes.AbstractFile;
import eu.svjatoslav.meviz.htmlindexer.metadata.fileTypes.Picture;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.List;

import static eu.svjatoslav.commons.file.FilePathParser.getFileNameWithoutExtension;
import static eu.svjatoslav.meviz.htmlindexer.AbstractIndexer.getVideoType;
import static eu.svjatoslav.meviz.htmlindexer.Utils.urlEncode;

public class MixedLayout implements Layout {

    public static final String encoding = "UTF-8";

    private final Logger logger = Logger.getLogger(MixedLayout.class);

    private StringBuffer dirHtml = new StringBuffer();

    private StringBuffer filesHtml = new StringBuffer();

    private String galleryTitle;

    private List<String> path;

    private AbstractIndexer indexer;

    private IndexingContext indexingContext;

    @Override
    public void enlistDirectory(final AbstractFile directory,
                                final DirectoryMetadata parentDirectoryMetadata) {
        dirHtml.append("<TR>");
        dirHtml.append("<TD><b><a href=\""
                + indexer.getDirectoryUrl(directory, indexingContext)
                + "\"><h2> ==&gt; " + directory.fileName + "</h2></a></b></TD>");
        dirHtml.append("</TR>\n");

    }

    @Override
    public void enlistFile(final AbstractFile file,
                           final DirectoryMetadata parentDirectoryMetadata) {

        final String fullFileName = file.fileName;

        filesHtml.append("<div style=\"clear: both;\"><a href=\""
                + indexingContext.getGlobalUrl()
                + urlEncode(indexingContext.getLocalUrl() + "/"
                + fullFileName) + "\">" + fullFileName + "</a>\n");
        filesHtml.append("&nbsp;&nbsp;("
                + FilePathParser.getFileSizeDescription(file.getFileLength())
                + ")</div><br/>\n");

    }

    @Override
    public void enlistImage(final Picture picture,
                            final DirectoryMetadata parentDirectoryMetadata) {
        try {
            enlistImageInternal(picture);
        } catch (final Exception e) {
            logger.error(
                    "Error generating thumbnails for image file: "
                            + e.toString()
                            + ", thumbnail generation skipped, enlisting as ordinary file instead.",
                    e);
            enlistFile(picture, parentDirectoryMetadata);
        }
    }

    private String enlistImageDimensions(final Picture picture) {
        final StringBuilder html = new StringBuilder();

        final List<Dimension> recommendedDimensions = Picture
                .getThumbnailDimensions(picture.getDimensions());

        int count = 0;
        for (final Dimension dimension : recommendedDimensions) {
            count++;
            if (count > 1)
                html.append(", ");
            html.append("<a href=\""
                    + indexer.getThumbnailUrl(picture, dimension,
                    indexingContext) + "\">");
            html.append(dimension.width + "x" + dimension.height);
            html.append("</a>");
        }

        return html.toString();
    }

    private void enlistImageInternal(final Picture picture) {

        filesHtml.append("<div style=\"float:left; margin: 10px;\">");

        // clickable image thumbnail
        filesHtml.append("<a href=\""
                + indexingContext.getGlobalUrl()
                + urlEncode(indexingContext.getLocalUrl() + "/"
                + picture.fileName) + "\">\n");

        filesHtml.append("<img border=\"0\" src=\""
                + getImageThumbnailUrl(picture) + "\"/></a><br/>\n");

        // file name
        filesHtml.append("<b>"
                + getFileNameWithoutExtension(picture.fileName)
                + "</b><br/>");

        // links to thumbnails of various sizes
        filesHtml.append("(" + enlistImageDimensions(picture) + ")\n");

        filesHtml.append("</div>");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFileNameSuffix() {
        return "";
    }

    /**
     * {@inheritDoc}
     *
     * @throws IOException
     */
    @Override
    public String getHtml(final boolean showTitle,
                          final boolean doHtmlHeaderAndFooter,
                          final DirectoryMetadata directory) {

        // assemble final page HTML
        final StringBuilder html = new StringBuilder();

        if (doHtmlHeaderAndFooter)
            appendHtmlHeader(html);

        if (showTitle)
            html.append("<CENTER><H1>" + galleryTitle + "</H1></CENTER>\n");

        html.append(showCurrentPath());
        html.append(listDirectories());
        html.append(directory.getHeaderMevizMetadataHtml());
        html.append(filesHtml.toString());

        if (doHtmlHeaderAndFooter)
            appendHtmlFooter(html);

        return html.toString();
    }

    private void appendHtmlFooter(StringBuilder html) {
        html.append("</font></BODY></HTML>\n");
    }

    private void appendHtmlHeader(StringBuilder html) {
        // page header
        html.append("<HTML>\n");
        html.append("<HEAD>\n");
        html.append("    <meta HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\"/>\n");
        html.append("</HEAD>\n");
        html.append("<BODY bgcolor=\"#000000\" alink=\"#50ffff\" link=\"#50ffff\" vlink=\"#ff50ff\" text=\"#ffffff\">\n");
        html.append("<!-- " + Constants.HTML_MAGIC_STRING + " -->\n");
        html.append("<!-- " + Constants.HTML_MEMO + " -->\n");
        html.append("<font color=\"#ffffff\">\n");
    }

    private String getImageThumbnailUrl(final Picture picture) {
        final Dimension imageSize = picture.getDimensions();

        Dimension listingDimension = new Dimension(1500, 450);

        if (listingDimension.width > imageSize.width)
            if (listingDimension.height > imageSize.height)
                listingDimension = imageSize;

        return indexer.getThumbnailUrl(picture, listingDimension,
                indexingContext);
    }

    @Override
    public void init(final String galleryTitle, final List<String> path,
                     final AbstractIndexer indexer, final IndexingContext indexingContext) {

        this.indexer = indexer;
        this.galleryTitle = galleryTitle;
        this.path = path;
        this.indexingContext = indexingContext;

        dirHtml = new StringBuffer();
        filesHtml = new StringBuffer();
    }

    private String listDirectories() {
        final StringBuilder html = new StringBuilder();
        html.append("<TABLE>\n");

        if (path.size() > 0) {
            html.append("<TR>");
            html.append("<TD><b><a href=\""
                    + indexer.getParentDirectoryUrl(indexingContext)
                    + "\"><h2> &lt;== parent directory</h2></a></b></TD>");
            html.append("</TR>\n");
        }

        html.append(dirHtml.toString());
        html.append("\n<TR><TD><br></TD></TR>\n");
        html.append("</TABLE>\n");

        return html.toString();
    }

    private String showCurrentPath() {
        final StringBuilder html = new StringBuilder();
        html.append("<CENTER><H2>");
        for (final String p : path)
            html.append(" / " + p);
        html.append("</H2></CENTER>\n");

        return html.toString();
    }

    @Override
    public void enlistVideo(AbstractFile file) {

        filesHtml.append("<div style=\"margin: 30px; clear: both;\">\n");

        // watchable video
        filesHtml.append("  <video width=\"640\" controls>\n");
        filesHtml.append("    <source src=\"" + getFileUrl(file) + "\" type=\""
                + getVideoType(file.getFileExtension()) + "\"/>\n");
        filesHtml.append("  </video>\n");

        filesHtml.append("  <br/>\n");

        // video title
        filesHtml.append("  Video: <b>"
                        + getFileNameWithoutExtension(file.fileName)
                        + "<b/>  &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");

        // video download link
        filesHtml.append("  <a href=\"" + getFileUrl(file) + "\">download</a>\n");

        filesHtml.append("</div>\n");

        filesHtml.append("\n");
    }


    private String getFileUrl(AbstractFile file) {
        return indexingContext.getGlobalUrl()
                    + urlEncode(indexingContext.getLocalUrl() + "/"
                    + file.fileName);
    }
}
