/*
 * Meviz - Various tools collection to work with multimedia.
 * Copyright (C) 2012 -- 2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 */

package eu.svjatoslav.meviz.htmlindexer.metadata;

import eu.svjatoslav.commons.file.FilePathParser;
import eu.svjatoslav.meviz.htmlindexer.AbstractIndexer;
import eu.svjatoslav.meviz.htmlindexer.FileSortComparator;
import eu.svjatoslav.meviz.htmlindexer.metadata.fileTypes.*;

import java.io.File;
import java.io.Serializable;
import java.util.*;

/**
 * Corresponds to single filesystem directory.
 */
public class DirectoryMetadata implements Serializable {

    private static final long serialVersionUID = 3526489102725366334L;

    private final HashMap<String, AbstractFile> fileNameToFileMap = new HashMap<>();

    public transient boolean changed;

    /**
     * Performs cleanup of directory model. In particular removes metadata for
     * images that no longer exist in the filesystem.
     */
    public void cleanup() {
        final Iterator<AbstractFile> fileIterator = fileNameToFileMap.values()
                .iterator();

        while (fileIterator.hasNext()) {
            final AbstractFile file = fileIterator.next();

            if (!file.isMetaInfoVerified())
                fileIterator.remove();
        }
    }

    public void ensureFileMetainfoIsUpToDate(final File parentDirectory,
                                             final File file) throws Exception {

        final String fileName = file.getName();

        if (fileNameToFileMap.containsKey(fileName)) {
            final boolean metainfoUpdated = fileNameToFileMap.get(fileName)
                    .ensureFileMetainfoIsUpToDate(parentDirectory);

            if (metainfoUpdated)
                changed = true;
        } else
            registerFile(parentDirectory, file);

    }

    private AbstractFile getAbstractFile(final File parentDirectory,
                                         final File file, final String fileName) throws Exception {

        if (file.isDirectory())
            return new DirectoryFile(parentDirectory, fileName);

        final String fileExtension = FilePathParser.getFileExtension(fileName)
                .toLowerCase();

        if (AbstractIndexer.isImage(fileExtension))
            try {
                return new Picture(parentDirectory, fileName);
            } catch (final Exception exception) {
                // in case image decoding failed, handle image as general file
                return new GeneralFile(parentDirectory, fileName);
            }

        if (fileName.endsWith(".meviz.org"))
            return new MevizSpecialFile(parentDirectory, fileName);

        if (fileName.endsWith(".meviz.html"))
            return new MevizSpecialFile(parentDirectory, fileName);

        return new GeneralFile(parentDirectory, fileName);
    }

    private AbstractFile getFile(final String fileName) {
        return fileNameToFileMap.get(fileName);
    }

    public Collection<AbstractFile> getFiles() {

        final ArrayList<AbstractFile> files = new ArrayList<>(
                fileNameToFileMap.values());

        Collections.sort(files, new FileSortComparator());

        return files;
    }

    public String getHeaderMevizMetadataHtml() {
        final AbstractFile headerFile = getFile("header.meviz.html");
        if (headerFile == null)
            return "";

        final MevizSpecialFile mevizSpecialFile = (MevizSpecialFile) headerFile;
        return mevizSpecialFile.getFileContent();
    }

    public Dimension getImageDimensions(final String fileName) {

        return ((Picture) getFile(fileName)).getDimensions();
    }

    private AbstractFile registerFile(final File parentDirectory, final File file)
            throws Exception {

        final String fileName = file.getName();

        final AbstractFile abstractFile = getAbstractFile(parentDirectory,
                file, fileName);

        fileNameToFileMap.put(fileName, abstractFile);

        changed = true;

        return abstractFile;
    }

    public void removeUnusedMetadataEntries() {

        new HashSet<>(fileNameToFileMap.keySet()).stream().filter(key -> !fileNameToFileMap.get(key).isMetaInfoVerified()).forEach(key -> {
            fileNameToFileMap.remove(key);
            changed = true;
        });

    }
}
