/*
 * Meviz - Various tools collection to work with multimedia.
 * Copyright (C) 2012 -- 2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 */

package eu.svjatoslav.meviz.htmlindexer.metadata.fileTypes;

import eu.svjatoslav.commons.file.FilePathParser;
import eu.svjatoslav.meviz.htmlindexer.Constants;
import eu.svjatoslav.meviz.htmlindexer.Utils;
import eu.svjatoslav.meviz.htmlindexer.metadata.Dimension;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

public class Picture extends AbstractFile {

    private static final long serialVersionUID = -4156533490858298387L;
    /**
     * Picture dimensions.
     */
    private Dimension dimensions;

    public Picture(final File parentDirectory, final String fileName)
            throws Exception {
        super(parentDirectory, fileName);
    }

    private static void ensureNonzeroImageArea(final java.awt.Dimension result) {
        if (result.width < 1)
            result.width = 1;
        if (result.height < 1)
            result.height = 1;
    }

    /**
     * Actual returned target thumbnail size will be adjusted from desired one
     * by attempting to generate as large as possible thumbnail, while not
     * exceeding provided thumbnail width and height, and preserving aspect
     * ratio of original image.
     */
    private static java.awt.Dimension getTargetThumbnailDimension(
            final java.awt.Dimension originalImageDimension,
            final java.awt.Dimension desiredTargetDimension) {

        final double thumbWidthToHeightRatio = (double) desiredTargetDimension.width
                / (double) desiredTargetDimension.height;
        final double inputImageWidthToHeightRatio = (double) originalImageDimension.width
                / (double) originalImageDimension.height;

        if (thumbWidthToHeightRatio < inputImageWidthToHeightRatio) {

            final java.awt.Dimension result = new java.awt.Dimension(
                    desiredTargetDimension.width,
                    (int) (desiredTargetDimension.width / inputImageWidthToHeightRatio));

            ensureNonzeroImageArea(result);

            return result;
        } else {

            final java.awt.Dimension result = new java.awt.Dimension(
                    (int) (desiredTargetDimension.height * inputImageWidthToHeightRatio),
                    desiredTargetDimension.height);

            ensureNonzeroImageArea(result);

            return result;
        }
    }

    /**
     * Propose list of thumbnail dimensions.
     */
    public static List<Dimension> getThumbnailDimensions(Dimension current) {
        final ArrayList<Dimension> result = new ArrayList<>();
        result.add(current);

        while (current.getArea() > 1000000) {
            current = current.getScaled(0.5d);
            result.add(current);
        }

        return result;

    }

    public static void makeThumbnail(final File inputFile,
                                     final File outputFile,
                                     final java.awt.Dimension preferredTargetDimensions) {

        try {

            final BufferedImage inputImage = Utils.getBufferedImage(inputFile);

            final java.awt.Dimension sourceImageDimension = new java.awt.Dimension(
                    inputImage.getWidth(), inputImage.getHeight());

            final java.awt.Dimension targetDimensions = getTargetThumbnailDimension(
                    sourceImageDimension, preferredTargetDimensions);

            final OutputStream out = new FileOutputStream(outputFile);

            final Image scaledImage = scaleImage(inputImage,
                    targetDimensions.width, targetDimensions.height);

            final BufferedImage bufferedImage = new BufferedImage(
                    scaledImage.getWidth(null), scaledImage.getHeight(null),
                    BufferedImage.TYPE_INT_RGB);

            final Graphics2D g = bufferedImage.createGraphics();
            g.drawImage(scaledImage, 0, 0, null);
            g.dispose();

            ImageIO.write(bufferedImage, "jpg", out);
            out.close();

        } catch (final Exception exception) {
            System.out.println(exception.toString());
            exception.printStackTrace();
        }
    }

    /**
     * @return new image scaled to desired dimensions
     */
    private static Image scaleImage(final Image srcImage, final int width,
                                    final int height) {

        final ImageFilter filter = new java.awt.image.AreaAveragingScaleFilter(
                width, height);

        final ImageProducer prod = new FilteredImageSource(
                srcImage.getSource(), filter);

        final Image newImage = Toolkit.getDefaultToolkit().createImage(prod);

        return new ImageIcon(newImage).getImage();
    }

    private String computeThumbnailHash(final Dimension targetDimension) {

        // compute new thumbnails hash number
        final String forMagicHash = fileName + " "
                + String.valueOf(targetDimension.width) + " "
                + String.valueOf(targetDimension.height) + " "
                + String.valueOf(getFileLength()) + " "
                + Constants.THUMBNAIL_VERSION;

        // System.out.println("Computing hash from: " + forMagicHash);
        return Utils.getStringCrcAsHex(forMagicHash);
    }

    public Dimension getDimensions() {
        return dimensions;
    }

    /**
     * Get thumbnail file name for this image and desired thumbnail dimensions
     * relative to designated thumbnails directory within parent directory.
     */
    public String getRelativeThumbnailFileName(final Dimension targetDimension) {
        return FilePathParser.getFileNameWithoutExtension(fileName) + " ("
                + computeThumbnailHash(targetDimension) + ").jpeg";
    }

    @Override
    public void updateFileMetainfo(final File parentDirectory) throws Exception {
        final BufferedImage image = Utils
                .getBufferedImage(getFile(parentDirectory));
        dimensions = new Dimension(image.getWidth(), image.getHeight());
    }

}
